@@ -18,7 +18,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <cheese-config.h>
 #endif
 
 #include <string.h>
@@ -139,8 +139,8 @@ static const EffectToPipelineDesc EFFECT_TO_PIPELINE_DESC [] =
   {CHEESE_WEBCAM_EFFECT_NOIR_BLANC, "videobalance saturation=0", YUV},
   {CHEESE_WEBCAM_EFFECT_SATURATION, "videobalance saturation=2", YUV},
   {CHEESE_WEBCAM_EFFECT_HULK, "videobalance saturation=1.5 hue=-0.5", YUV},
-  {CHEESE_WEBCAM_EFFECT_VERTICAL_FLIP, "videoflip method=4", YUV},
-  {CHEESE_WEBCAM_EFFECT_HORIZONTAL_FLIP, "videoflip method=5", YUV},
+  {CHEESE_WEBCAM_EFFECT_VERTICAL_FLIP, "videoflip method=5", YUV},
+  {CHEESE_WEBCAM_EFFECT_HORIZONTAL_FLIP, "videoflip method=4", YUV},
   {CHEESE_WEBCAM_EFFECT_SHAGADELIC, "shagadelictv", RGB},
   {CHEESE_WEBCAM_EFFECT_VERTIGO, "vertigotv", RGB},
   {CHEESE_WEBCAM_EFFECT_EDGE, "edgetv", RGB},
@@ -478,9 +478,19 @@ cheese_webcam_get_webcam_device_data (CheeseWebcamDevice *webcam_device)
   i = 0;
   while (!pipeline_works && (i < G_N_ELEMENTS (GSTREAMER_VIDEO_SOURCES)))
   {
-    pipeline_desc = g_strdup_printf ("%s name=source device=%s ! fakesink", 
-                                     GSTREAMER_VIDEO_SOURCES[i],
-                                     webcam_device->video_device);
+    /* NOTE queue-size=2 can be removed as soon as gst-plugins-good 0.10.7 comes out */
+    if (strcmp (GSTREAMER_VIDEO_SOURCES[i], "v4l2src") == 0)
+    {
+      pipeline_desc = g_strdup_printf ("%s name=source device=%s queue-size=2 ! fakesink", 
+                                       GSTREAMER_VIDEO_SOURCES[i],
+                                       webcam_device->video_device);
+    }
+    else
+    {
+      pipeline_desc = g_strdup_printf ("%s name=source device=%s ! fakesink", 
+                                       GSTREAMER_VIDEO_SOURCES[i],
+                                       webcam_device->video_device);
+    }
     err = NULL;
     pipeline = gst_parse_launch (pipeline_desc, &err);
     if ((pipeline != NULL) && (err == NULL))
@@ -610,14 +620,29 @@ cheese_webcam_create_webcam_source_bin (CheeseWebcam *webcam)
       }
     }
 
-    webcam_input = g_strdup_printf ("%s name=video_source device=%s ! %s,width=%d,height=%d,framerate=%d/%d ! identity",
-                                    selected_webcam->gstreamer_src,
-                                    selected_webcam->video_device,
-                                    format->mimetype,
-                                    format->width,
-                                    format->height,
-                                    framerate_numerator,
-                                    framerate_denominator);
+    /* NOTE queue-size=2 can be removed as soon as gst-plugins-good 0.10.7 comes out */
+    if (strcmp (selected_webcam->gstreamer_src, "v4l2src") == 0)
+    {
+      webcam_input = g_strdup_printf ("%s name=video_source device=%s queue-size=2 ! %s,width=%d,height=%d,framerate=%d/%d ! identity",
+                                      selected_webcam->gstreamer_src,
+                                      selected_webcam->video_device,
+                                      format->mimetype,
+                                      format->width,
+                                      format->height,
+                                      framerate_numerator,
+                                      framerate_denominator);
+    }
+    else
+    {
+      webcam_input = g_strdup_printf ("%s name=video_source device=%s ! %s,width=%d,height=%d,framerate=%d/%d ! identity",
+                                      selected_webcam->gstreamer_src,
+                                      selected_webcam->video_device,
+                                      format->mimetype,
+                                      format->width,
+                                      format->height,
+                                      framerate_numerator,
+                                      framerate_denominator);
+    }
     g_print ("%s\n", webcam_input);
 
     priv->webcam_source_bin = gst_parse_bin_from_description (webcam_input,
@@ -711,9 +736,7 @@ cheese_webcam_create_photo_save_bin (CheeseWebcam *webcam)
   caps = gst_caps_new_simple ("video/x-raw-rgb",
       			      "bpp", G_TYPE_INT, 24,
       			      "depth", G_TYPE_INT, 24,
-      			      "red_mask",   G_TYPE_INT, 0xff0000, /* enforce rgb */
-      			      "green_mask", G_TYPE_INT, 0x00ff00,
-      			      "blue_mask",  G_TYPE_INT, 0x0000ff, NULL);
+			      NULL);
   ok = gst_element_link_filtered (csp_photo_save_bin, priv->photo_sink, caps);
   gst_caps_unref (caps);
 
