@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <cheese-config.h>
 #endif
 
 #include <glib.h>
